@@ -25,6 +25,7 @@
 #include <string.h>
 #include <stdio.h>
 #include <syslog.h>
+#include <stdarg.h>
 
 #include "dataxfer.h"
 #include "readconfig.h"
@@ -157,14 +158,14 @@ out:
 }
 
 char *
-find_str(char *name, enum str_type *type)
+find_str(const char *name, enum str_type *type)
 {
     struct longstr_s *longstr = longstrs;
 
     while (longstr) {
 	if (strcmp(name, longstr->name) == 0) {
 	    *type = longstr->type;
-	    return longstr->str;
+	    return strdup(longstr->str);
 	}
 	longstr = longstr->next;
     }
@@ -203,6 +204,15 @@ struct tracefile_s
     struct tracefile_s *next;
 };
 
+#ifdef USE_RS485_FEATURE
+struct rs485conf_s
+{
+    char *name;
+    struct serial_rs485 conf;
+    struct rs485conf_s *next;
+};
+#endif
+
 /* All the tracefiles in the system. */
 struct tracefile_s *tracefiles = NULL;
 
@@ -237,13 +247,13 @@ handle_tracefile(char *name, char *fname)
 }
 
 char *
-find_tracefile(char *name)
+find_tracefile(const char *name)
 {
     struct tracefile_s *tracefile = tracefiles;
 
     while (tracefile) {
 	if (strcmp(name, tracefile->name) == 0)
-	    return tracefile->str;
+	    return strdup(tracefile->str);
 	tracefile = tracefile->next;
     }
     syslog(LOG_ERR, "Tracefile %s not found, it will be ignored", name);
@@ -264,13 +274,128 @@ free_tracefiles(void)
     }
 }
 
+#ifdef USE_RS485_FEATURE
+/* All the RS485 configs in the system. */
+struct rs485conf_s *rs485confs = NULL;
+
+static void
+handle_rs485conf(char *name, char *str)
+{
+    struct rs485conf_s *new_rs485conf;
+    uint8_t rts_on_send, rx_during_tx;
+
+    new_rs485conf = malloc(sizeof(*new_rs485conf));
+    if (!new_rs485conf) {
+	syslog(LOG_ERR, "Out of memory handling rs485 config on %d", lineno);
+	return;
+    }
+
+    new_rs485conf->name = strdup(name);
+    if (!new_rs485conf->name) {
+	syslog(LOG_ERR, "Out of memory handling rs485 config on %d", lineno);
+	free(new_rs485conf);
+	return;
+    }
+
+    if (sscanf(str, "%u:%u:%1hhu:%1hhu",
+               &new_rs485conf->conf.delay_rts_before_send,
+               &new_rs485conf->conf.delay_rts_after_send,
+               &rts_on_send,
+               &rx_during_tx) != 4) {
+	syslog(LOG_ERR, "Couldn't parse RS485 config on %d", lineno);
+	return;
+    }
+
+    /* check, if flags have values 0 or 1 */
+    if (rts_on_send > 1) {
+	syslog(LOG_ERR, "RTS_ON_SEND parameter can be 0 or 1 on %d", lineno);
+	return;
+    }
+
+    if (rx_during_tx > 1) {
+	syslog(LOG_ERR, "RX_DURING_TX parameter can be 0 or 1 on %d", lineno);
+	return;
+    }
+
+    new_rs485conf->conf.flags = SER_RS485_ENABLED;
+
+    if (rts_on_send) {
+        new_rs485conf->conf.flags |= SER_RS485_RTS_ON_SEND;
+    } else {
+        new_rs485conf->conf.flags |= SER_RS485_RTS_AFTER_SEND;
+    }
+
+    if (rx_during_tx) {
+        new_rs485conf->conf.flags |= SER_RS485_RX_DURING_TX;
+    }
+
+    new_rs485conf->next = rs485confs;
+    rs485confs = new_rs485conf;
+}
+
+struct serial_rs485 *
+find_rs485conf(const char *name)
+{
+    struct rs485conf_s *new_rs485conf = rs485confs;
+
+    while (new_rs485conf) {
+        if (strcmp(name, new_rs485conf->name) == 0)
+            return &new_rs485conf->conf;
+        new_rs485conf = new_rs485conf->next;
+    }
+    syslog(LOG_ERR, "RS485 configuration %s not found, it will be ignored", name);
+    return NULL;
+}
+
+static void
+free_rs485confs(void)
+{
+    struct rs485conf_s *rs485conf;
+
+    while (rs485confs) {
+        rs485conf = rs485confs;
+        rs485confs = rs485confs->next;
+        free(rs485conf->name);
+        free(rs485conf);
+    }
+}
+#endif
+
+static int
+startswith(char *str, const char *test, char **strtok_data)
+{
+    int len = strlen(test);
+
+    if ((strncmp(str, test, len) == 0) && (str[len] == ':')) {
+	strtok_r(str, ":", strtok_data);
+	return 1;
+    }
+    return 0;
+}
+
+static int
+syslog_eprint(struct absout *e, const char *str, ...)
+{
+    va_list ap;
+    char buf[1024];
+
+    va_start(ap, str);
+    vsnprintf(buf, sizeof(buf), str, ap);
+    va_end(ap);
+    syslog(LOG_ERR, "%s on line %d", buf, *((int *) e->data));
+    return 0;
+}
+
+static struct absout syslog_eout = {
+    .out = syslog_eprint,
+    .data = &lineno
+};
 
 void
 handle_config_line(char *inbuf)
 {
-    char *portnum, *state, *timeout, *devname, *devcfg;
+    char *portnum, *state, *timeout, *devname, *devcfg, *comma;
     char *strtok_data = NULL;
-    char *errstr;
 
     lineno++;
 
@@ -287,13 +412,7 @@ handle_config_line(char *inbuf)
 	return;
     }
 
-    portnum = strtok_r(inbuf, ":", &strtok_data);
-    if (portnum == NULL) {
-	/* An empty line is ok. */
-	return;
-    }
-
-    if (strcmp(portnum, "CONTROLPORT") == 0) {
+    if (startswith(inbuf, "CONTROLPORT", &strtok_data)) {
 	if (config_port)
 	    /*
 	     * The control port has already been configured either on the
@@ -308,7 +427,7 @@ handle_config_line(char *inbuf)
 	return;
     }
 
-    if (strcmp(portnum, "BANNER") == 0) {
+    if (startswith(inbuf, "BANNER", &strtok_data)) {
 	char *name = strtok_r(NULL, ":", &strtok_data);
 	char *str = strtok_r(NULL, "\n", &strtok_data);
 	if (name == NULL) {
@@ -319,7 +438,7 @@ handle_config_line(char *inbuf)
 	return;
     }
 
-    if (strcmp(portnum, "SIGNATURE") == 0) {
+    if (startswith(inbuf, "SIGNATURE", &strtok_data)) {
 	char *name = strtok_r(NULL, ":", &strtok_data);
 	char *str = strtok_r(NULL, "\n", &strtok_data);
 	if (name == NULL) {
@@ -330,7 +449,24 @@ handle_config_line(char *inbuf)
 	return;
     }
 
-    if (strcmp(portnum, "OPENSTR") == 0) {
+#ifdef USE_RS485_FEATURE
+    if (startswith(inbuf, "RS485CONF", &strtok_data)) {
+        char *name = strtok_r(NULL, ":", &strtok_data);
+        char *str = strtok_r(NULL, "\n", &strtok_data);
+        if (name == NULL) {
+            syslog(LOG_ERR, "No signature given on line %d", lineno);
+            return;
+        }
+        if ((str == NULL) || (strlen(str) == 0)) {
+            syslog(LOG_ERR, "No RS485 configuration given on line %d", lineno);
+            return;
+        }
+        handle_rs485conf(name, str);
+        return;
+    }
+#endif
+
+    if (startswith(inbuf, "OPENSTR", &strtok_data)) {
 	char *name = strtok_r(NULL, ":", &strtok_data);
 	char *str = strtok_r(NULL, "\n", &strtok_data);
 	if (name == NULL) {
@@ -341,7 +477,7 @@ handle_config_line(char *inbuf)
 	return;
     }
 
-    if (strcmp(portnum, "CLOSESTR") == 0) {
+    if (startswith(inbuf, "CLOSESTR", &strtok_data)) {
 	char *name = strtok_r(NULL, ":", &strtok_data);
 	char *str = strtok_r(NULL, "\n", &strtok_data);
 	if (name == NULL) {
@@ -352,7 +488,7 @@ handle_config_line(char *inbuf)
 	return;
     }
 
-    if (strcmp(portnum, "TRACEFILE") == 0) {
+    if (startswith(inbuf, "TRACEFILE", &strtok_data)) {
 	char *name = strtok_r(NULL, ":", &strtok_data);
 	char *str = strtok_r(NULL, "\n", &strtok_data);
 	if (name == NULL) {
@@ -367,6 +503,21 @@ handle_config_line(char *inbuf)
 	return;
     }
 
+    comma = strchr(inbuf, ',');
+    if (comma) {
+	if (!strtok_r(comma, ":", &strtok_data)) {
+	    syslog(LOG_ERR, "Invalid port on line %d", lineno);
+	    return;
+	}
+	portnum = inbuf;
+    } else {
+	portnum = strtok_r(inbuf, ":", &strtok_data);
+	if (portnum == NULL) {
+	    /* An empty line is ok. */
+	    return;
+	}
+    }
+
     state = strtok_r(NULL, ":", &strtok_data);
     if (state == NULL) {
 	syslog(LOG_ERR, "No state given on line %d", lineno);
@@ -391,11 +542,8 @@ handle_config_line(char *inbuf)
 	devcfg = "";
     }
 
-    errstr = portconfig(portnum, state, timeout, devname, devcfg,
-			config_num);
-    if (errstr != NULL) {
-	syslog(LOG_ERR, "Error on line %d, %s", lineno, errstr);
-    }
+    portconfig(&syslog_eout, portnum, state, timeout, devname, devcfg,
+	       config_num);
 }
 
 /* Read the specified configuration file and call the routine to
@@ -417,6 +565,9 @@ readconfig(char *filename)
 
     free_longstrs();
     free_tracefiles();
+#ifdef USE_RS485_FEATURE
+    free_rs485confs();
+#endif
 
     config_num++;
 
