@@ -2,7 +2,7 @@
 #ifndef _SER2NET_TELNET_H
 #define _SER2NET_TELNET_H
 
-#include <buffer.h>
+#include "buffer.h"
 
 /* Telnet commands */
 #define TN_BREAK 243
@@ -102,11 +102,4 @@ void telnet_init(telnet_data_t *td,
 		 unsigned char *init_seq,
 		 int init_seq_len);
 
-/* Set to true if we are supposed to do CISCO IOS baud rates instead
-   of RFC2217 ones. */
-extern int cisco_ios_baud_rates;
-
-/* The default rfc2217 signature if none is provided. */
-extern char *rfc2217_signature;
-
 #endif /* _SER2NET_TELNET_H */
